package com.spectus.iap.spectus_in_app_purchase

import android.app.Activity
import android.content.Context
import android.util.Log
import com.android.billingclient.api.*


class IAPHelper {

    // Product Purchases param
    var context: Context? = null
    var productId: String = ""
    var isSubscription: Boolean = false
    var billingClient: BillingClient? = null
    var mActivity_PRODUCT: Activity? = null

    var skuProductDetails: SkuDetails? = null
    var storedProductDataList: ArrayList<SkuDetails> = ArrayList()
    var productList: ArrayList<Map<String, String>> = arrayListOf()
    var productIdForPrice: String = ""

    var purchaseCallBack: ((errorMessage: String?, purchaseObject: Purchase?, purchaseID: String?) -> Unit)? =
        null
    var skuDataListCallBack: ((errorMessage: String?, skuDetails: String?) -> Unit)? =
        null

    var purchaseCallBackForPrice: ((itemPrice: String) -> Unit)? =
        null
    var calledOnce = false
    var purchaseProcessAlreadyBegun = false

    fun startInAppPurchaseWith(
        mActivityObj: Activity,
        contextObj: Context,
        productIdentifier: String,
        isSubscription: Boolean,
        callbackproduct: (errorMessage: String?, purchaseObject: Purchase?, purchaseID: String?) -> Unit
    ) {
        print("==>> startInAppPurchaseWith")
        productId = productIdentifier
        context = contextObj
        mActivity_PRODUCT = mActivityObj
        purchaseCallBack = callbackproduct
        skuDataListCallBack = null
        this.isSubscription = isSubscription

        billingClient = null
        this.setUpBillingClient(contextObj)
    }


    fun setUpBillingClient(context: Context) {
        // do your stuff
        billingClient = BillingClient.newBuilder(context)
            .enablePendingPurchases()
            .setListener(purchaseUpdateListener)
            .build()
        this.startConnection(context)
    }

    fun startConnection(context: Context) {
        billingClient?.startConnection(object : BillingClientStateListener {
            override fun onBillingSetupFinished(billingResult: BillingResult) {
                if (billingResult.responseCode == BillingClient.BillingResponseCode.OK) {
                    // The BillingClient is ready. You can query purchases here.
                    Log.v("TAG_INAPP", "Setup Billing Done")
                    queryAvaliableProducts(context)
                }
            }

            override fun onBillingServiceDisconnected() {

            }
        })
    }

    fun queryAvaliableProducts(context: Context) {
        Log.v("TAG_INAPP productId", "" + productId)
        var skuList = listOf(productId)

        var skuType  =  BillingClient.SkuType.INAPP
        if (isSubscription) {
            skuType = BillingClient.SkuType.SUBS
        }

        val params = SkuDetailsParams.newBuilder()
            .setSkusList(skuList)
            .setType(skuType)
            .build()

        billingClient?.querySkuDetailsAsync(params) { billingResult, skuDetailsList
            ->
            Log.e("INSIDE", "" + skuDetailsList!!.size.toString())
            Log.e("RESULT Code => ", "" + billingResult.responseCode.toString())
            // Process the result.
            if (billingResult.responseCode == BillingClient.BillingResponseCode.OK && !skuDetailsList.isNullOrEmpty()) {
                storedProductDataList.clear()
                storedProductDataList.addAll(skuDetailsList)

                var details: String = ""
                var counter = 0
                for (skuDetails in skuDetailsList) {
                    counter++
                    Log.v("TAG_INAPP", "skuDetailsList : ${skuDetailsList}")
                    //This list should contain the products added above
                    if (context != null) {
                        Log.e("context not null", "context not null")
                    } else {
                        Log.e("context null", "context null")
                    }


                    val product: MutableMap<String, String> = HashMap()
                    product["currency"] = skuDetails.priceCurrencyCode
                    product["amount"] = skuDetails.price
                    product["title"] = skuDetails.title
                    product["description"] = skuDetails.description
                    product["productID"] = skuDetails.sku

                    productList.add(product)

                    var productTitle: String = skuDetails.title
                    val index = productTitle.indexOf(" (")
                    if (index != -1) {
                        productTitle = productTitle.substring(0, index)
                    }

                    if (!details.isEmpty()) {
                        val sb = StringBuilder()

                        var info = skuDetails.price + "&&" + skuDetails.priceCurrencyCode +
                                "&&" + productTitle +
                                "&&" + skuDetails.description +
                                "&&" + skuDetails.sku

                        sb.append(details).append("&^&").append(info)
                        details = sb.toString()

                    } else {
                        details = skuDetails.price + "&&" + skuDetails.priceCurrencyCode +
                                "&&" + productTitle +
                                "&&" + skuDetails.description +
                                "&&" + skuDetails.sku
                    }

                    if (counter == skuDetailsList.size) {
                        Log.v("TAG_INAPP", "updateUI Called")
                        updateUI(skuDetails, context)
                    }

                }
            }
        }
    }

    fun updateUI(skuDetails: SkuDetails?, context: Context) {
        skuDetails?.let {
            this.skuProductDetails = it
        }
        Log.e("herer null", "geterer null")
        skuProductDetails?.let {
            val billingFlowParams = BillingFlowParams.newBuilder()
                .setSkuDetails(it)
                .build()
            Log.e("billingFlowParams", "" + billingFlowParams)

            if (mActivity_PRODUCT != null) {
                Log.e("act not null", "act not null")
            } else {
                Log.e("act  null", "act  null")
            }
            billingClient?.launchBillingFlow(mActivity_PRODUCT!!, billingFlowParams)?.responseCode
        } ?: noSKUMessage()

    }

    private fun noSKUMessage() {
        Log.e("no sku msg", "no sku msg")
    }

    val purchaseUpdateListener = PurchasesUpdatedListener { billingResult, purchases ->
        Log.v("TAG_INAPP", "billingResult responseCode : ${billingResult.responseCode}")
        if (purchaseProcessAlreadyBegun == false) {
            Log.v("TAG_INAPP", "Purchase Not Begun")
            purchaseProcessAlreadyBegun = true
            if (billingResult.responseCode == Constant.OK && purchases != null) {
                var purchaseData: Purchase? = null
                for (purchase in purchases) {
                    Log.v("TAG_INAPP", "Inside Loop : "+ purchase.purchaseTime.toString())
                    purchaseData = purchase
                }

                Log.v("TAG_INAPP", "SUBSCRIPTION => " + this.isSubscription.toString())
                if (this.isSubscription) {
                    Log.v("TAG_INAPP", "Called Non-Consumable")
                    handleNonConcumablePurchase(purchaseData!!)
                } else {
                    Log.v("TAG_INAPP", "Called Consumable")
                    handleConsumedPurchases(purchaseData!!)
                }
            } else if (billingResult.responseCode != null) {
                Log.v("TAG_INAPP", "billingResult responseCode Other Than OK : ${billingResult.responseCode}")
                if (purchaseCallBack != null && calledOnce == false) {
                    calledOnce = true
                    purchaseCallBack!!(billingResult.responseCode.toString(), null, null)
                }
            }
         else {
                // Handle any other error codes.
                Log.v("TAG_INAPP", "billingResult responseCode Not Matched : ${billingResult.responseCode}")
            }

        } else {
            Log.v("TAG_INAPP", "Purchase Already Begun")
        }
    }

    private fun handleConsumedPurchases(purchase: Purchase) {
        Log.d("TAG_INAPP", "handleConsumablePurchasesAsync foreach it is $purchase")

        val orderId = purchase.orderId

        val params =
            ConsumeParams.newBuilder().setPurchaseToken(purchase.purchaseToken).build()
        billingClient?.consumeAsync(params) { billingResult, purchaseToken ->
            when (billingResult.responseCode) {
                Constant.OK -> {
                    // Update the appropriate tables/databases to grant user the items
                    val orderID = orderId
                    Log.e("Callback of Transaction", "Success => " + orderID!!)


                    if (purchaseCallBack != null && skuDataListCallBack == null && calledOnce == false)
                        calledOnce = true
                    purchaseCallBack!!(null, purchase, orderID)

                }
            }
        }
    }

    private fun handleNonConcumablePurchase(purchase: Purchase) {
        Log.v("TAG_INAPP", "handlePurchase : ${purchase}")
        if (purchase.purchaseState == Purchase.PurchaseState.PURCHASED) {
            if (!purchase.isAcknowledged) {
                val acknowledgePurchaseParams = AcknowledgePurchaseParams.newBuilder()
                    .setPurchaseToken(purchase.purchaseToken).build()
                billingClient?.acknowledgePurchase(acknowledgePurchaseParams) { billingResult ->

                    when (billingResult.responseCode) {
                        Constant.OK -> {
                            // Update the appropriate tables/databases to grant user the items
                            var orderID = purchase.orderId
                            Log.e("Callback of Transaction", "Success => " + orderID!!)

                            if (purchaseCallBack != null && skuDataListCallBack == null && calledOnce == false)
                                calledOnce = true
                            purchaseCallBack!!(null, purchase, orderID)

                        }
                    }

                    val billingResponseCode = billingResult.responseCode
                    val billingDebugMessage = billingResult.debugMessage
                    Log.v("TAG_INAPP", "response code: $billingResponseCode")
                    Log.v("TAG_INAPP", "debugMessage : $billingDebugMessage")
                }
            }
        }
    }

    class Constant {

        companion object {

            // App Product ID from Play Console
            var PRODUCT_ID = "com.monthly.inapp"
            var PRODUCT_ID_2 = "data_value_product"

            /*
     * Result code of BillingClient*/
            //Success

            // Codes for Billiing Details
            var SERVICE_TIMEOUT = -3
            var FEATURE_NOT_SUPPORTED = -2
            var SERVICE_DISCONNECTED = -1
            var OK = 0
            var USER_CANCELED = 1
            var SERVICE_UNAVAILABLE = 2
            var BILLING_UNAVAILABLE = 3
            var ITEM_UNAVAILABLE = 4
            var DEVELOPER_ERROR = 5
            var ERROR = 6
            var ITEM_ALREADY_OWNED = 7
            var ITEM_NOT_OWNED = 8

        }

    }

    fun startInAppPurchaseWithPriceFromName(
        mActivityObj: Activity,
        contextObj: Context,
        productIdentifier: String,
        isSubscription: Boolean,
        callbackproduct: (itemPrice: String) -> Unit
    ) {
        print("==>> startInAppPurchaseWith")
        productIdForPrice = productIdentifier
        context = contextObj
        mActivity_PRODUCT = mActivityObj
        purchaseCallBackForPrice = callbackproduct
        this.isSubscription = isSubscription

        billingClient = null
        this.setUpBillingClientPriceFromName(contextObj)
    }

    fun setUpBillingClientPriceFromName(context: Context) {
        // do your stuff
        billingClient = BillingClient.newBuilder(context)
            .enablePendingPurchases()
            .setListener(purchaseUpdateListenerForName)
            .build()
        this.startConnectionPriceFromName(context)
    }

    fun startConnectionPriceFromName(context: Context) {
        billingClient?.startConnection(object : BillingClientStateListener {
            override fun onBillingSetupFinished(billingResult: BillingResult) {
                if (billingResult.responseCode == BillingClient.BillingResponseCode.OK) {
                    // The BillingClient is ready. You can query purchases here.
                    Log.v("TAG_INAPP", "Setup Billing Done")
                    getProductPriceFromName()
                }
            }

            override fun onBillingServiceDisconnected() {

            }
        })
    }

    fun getProductPriceFromName() {
        var skuList = listOf(productIdForPrice)

        var skuType  =  BillingClient.SkuType.INAPP
        if (isSubscription) {
            skuType = BillingClient.SkuType.SUBS
        }

        val params = SkuDetailsParams.newBuilder()
            .setSkusList(skuList)
            .setType(skuType)
            .build()

        billingClient?.querySkuDetailsAsync(params) { billingResult, skuDetailsList
            ->
            Log.e("RESULT Code => ", "" + billingResult.responseCode.toString())
            // Process the result.
            if (billingResult.responseCode == BillingClient.BillingResponseCode.OK && !skuDetailsList.isNullOrEmpty()) {
                purchaseCallBackForPrice!!.invoke(skuDetailsList.toString())
            }
        }
    }

    val purchaseUpdateListenerForName = PurchasesUpdatedListener { billingResult, purchases ->
        Log.v("TAG_INAPP", "billingResult responseCode : ${billingResult.responseCode}")
    }

}